@@ -28,38 +28,27 @@
 
 """Collection of utility classes and functions to aid the solace_* modules."""
 
-import re
 import traceback
 import logging
 import json
 import time
-import os
-import sys
-from distutils.util import strtobool
-
+import ansible.module_utils.network.solace.solace_common as sc
 HAS_IMPORT_ERROR = False
+IMPORT_ERR_TRACEBACK = None
 try:
+    from inspect import signature
+    import ansible.module_utils.network.solace.solace_cloud_utils as scu
     from ansible.errors import AnsibleError
-    from json.decoder import JSONDecodeError
     import requests
-    import xmltodict
 except ImportError:
     HAS_IMPORT_ERROR = True
     IMPORT_ERR_TRACEBACK = traceback.format_exc()
 
-# check python version
-_PY3_MIN = sys.version_info[:2] >= (3, 6)
-if not _PY3_MIN:
-    print(
-        '\n{"failed": true, '
-        '"msg": "failed: ansible-solace requires a minimum of Python3 version 3.6. Current version: %s. Hint: set ANSIBLE_PYTHON_INTERPRETER=path-to-python-3"}' % (''.join(sys.version.splitlines()))
-    )
-    sys.exit(1)
 
 """ Default Whitelist Keys """
 DEFAULT_WHITELIST_KEYS = ['password']
 
-""" Solace Cloud reources """
+""" Solace Cloud resources """
 SOLACE_CLOUD_API_SERVICES_BASE_PATH = 'https://api.solace.cloud/api/v0/services'
 SOLACE_CLOUD_REQUESTS = 'requests'
 SOLACE_CLOUD_CLIENT_PROFILE_REQUESTS = 'clientProfileRequests'
@@ -100,24 +89,6 @@ CERT_AUTHORITIES = 'certAuthorities'
 MQTT_SESSIONS = 'mqttSessions'
 MQTT_SESSION_SUBSCRIPTIONS = 'subscriptions'
 
-################################################################################################
-# initialize logging
-ENABLE_LOGGING = False  # False to disable
-enableLoggingEnvVal = os.getenv('ANSIBLE_SOLACE_ENABLE_LOGGING')
-if enableLoggingEnvVal is not None and enableLoggingEnvVal != '':
-    try:
-        ENABLE_LOGGING = bool(strtobool(enableLoggingEnvVal))
-    except ValueError:
-        raise ValueError("failed: invalid value for env var: 'ANSIBLE_SOLACE_ENABLE_LOGGING={}'. use 'true' or 'false' instead.".format(enableLoggingEnvVal))
-
-if ENABLE_LOGGING:
-    logging.basicConfig(filename='ansible-solace.log',
-                        level=logging.DEBUG,
-                        format='%(asctime)s - %(name)s - %(levelname)s - %(funcName)s(): %(message)s')
-    logging.info('Module start #############################################################################################')
-
-################################################################################################
-
 
 class SolaceConfig(object):
 
@@ -142,11 +113,7 @@ class SolaceConfig(object):
 class SolaceTask:
 
     def __init__(self, module):
-        if HAS_IMPORT_ERROR:
-            exceptiondata = traceback.format_exc().splitlines()
-            exceptionarray = [exceptiondata[-1]] + exceptiondata[1:-1]
-            module.fail_json(msg="failed: Missing module: %s" % exceptionarray[0], exception=IMPORT_ERR_TRACEBACK)
-
+        sc.module_fail_on_import_error(module, HAS_IMPORT_ERROR, IMPORT_ERR_TRACEBACK)
         self.module = module
         solace_cloud_api_token = self.module.params.get('solace_cloud_api_token', None)
         solace_cloud_service_id = self.module.params.get('solace_cloud_service_id', None)
@@ -154,8 +121,8 @@ class SolaceTask:
         ok = ((solace_cloud_api_token and solace_cloud_service_id)
               or (not solace_cloud_api_token and not solace_cloud_service_id))
         if not ok:
-            result = dict(changed=False, response=dict())
-            msg = "failed: must provide either both or none for Solace Cloud: solace_cloud_api_token={}, solace_cloud_service_id={}.".format(solace_cloud_api_token, solace_cloud_service_id)
+            result = dict(changed=False, rc=1)
+            msg = "must provide either both or none for Solace Cloud: solace_cloud_api_token={}, solace_cloud_service_id={}.".format(solace_cloud_api_token, solace_cloud_service_id)
             self.module.fail_json(msg=msg, **result)
 
         if ok and solace_cloud_api_token and solace_cloud_service_id:
@@ -191,7 +158,7 @@ class SolaceTask:
 
         if settings:
             # jinja treats everything as a string, so cast ints and floats
-            settings = _type_conversion(settings)
+            settings = sc.type_conversion(settings, is_broker_solace_cloud(self.solace_config))
 
         ok, resp = self.get_func(self.solace_config, *(self.get_args() + [self.lookup_item()]))
 
@@ -256,12 +223,20 @@ class SolaceTask:
 
                     if len(changed_keys):
                         delta_settings = {key: settings[key] for key in changed_keys}
-                        crud_args.append(delta_settings)
                         if not self.module.check_mode:
+                            crud_args.append(delta_settings)
+                            # Note:
+                            # only add current_settings for modules that support it.
+                            # parameter must be called 'current_settings'
+                            update_func_signature = signature(self.update_func, follow_wrapped=False)
+                            for param in update_func_signature.parameters.values():
+                                if param.name == "current_settings":
+                                    crud_args.append(current_configuration)
+
                             ok, resp = self.update_func(self.solace_config, *crud_args)
                             result['response'] = resp
                             if not ok:
-                                self.module.fail_json(msg=resp, **result)
+                                self.module.fail_json(msg="Error in update_func(). Pls raise an issue.", **result)
                         result['delta'] = delta_settings
                         result['changed'] = True
                 else:
@@ -380,8 +355,8 @@ class SolaceTask:
                             params=None
                 )
 
-                if ENABLE_LOGGING:
-                    log_http_roundtrip(resp)
+                if sc.ENABLE_LOGGING:
+                    sc.log_http_roundtrip(resp)
 
                 if resp.status_code != 200:
                     return False, parse_bad_response(resp)
@@ -390,7 +365,7 @@ class SolaceTask:
                     if "data" in body.keys():
                         result_list.extend(body['data'])
 
-            except requests.exceptions.ConnectionError as e:
+            except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as e:
                 return False, str(e)
 
             if "meta" not in body:
@@ -417,15 +392,15 @@ def arg_spec_broker():
         secure_connection=dict(type='bool', default=False),
         username=dict(type='str', default='admin'),
         password=dict(type='str', default='admin', no_log=True),
-        timeout=dict(type='int', default='10', require=False),
+        timeout=dict(type='int', default='10', required=False),
         x_broker=dict(type='str', default='')
     )
 
 
 def arg_spec_solace_cloud_config():
     return dict(
-        solace_cloud_api_token=dict(type='str', require=False, no_log=True, default=None),
-        solace_cloud_service_id=dict(type='str', require=False, default=None)
+        solace_cloud_api_token=dict(type='str', required=False, no_log=True, default=None),
+        solace_cloud_service_id=dict(type='str', required=False, default=None)
     )
 
 
@@ -443,13 +418,13 @@ def arg_spec_virtual_router():
 
 def arg_spec_settings():
     return dict(
-        settings=dict(type='dict', require=False)
+        settings=dict(type='dict', required=False)
     )
 
 
 def arg_spec_semp_version():
     return dict(
-        semp_version=dict(type='str', require=True)
+        semp_version=dict(type='str', required=True)
     )
 
 
@@ -476,7 +451,7 @@ def arg_spec_get_list():
     return dict(
         api=dict(type='str', default='config', choices=['config', 'monitor']),
         query_params=dict(type='dict',
-                          require=False,
+                          required=False,
                           options=dict(
                             select=dict(type='list', default=[], elements='str'),
                             where=dict(type='list', default=[], elements='str')
@@ -506,18 +481,6 @@ def _build_config_dict(resp, key):
     return d
 
 
-def _type_conversion(d):
-    for k, i in d.items():
-        t = type(i)
-        if (t == str) and re.search(r'^[0-9]+$', i):
-            d[k] = int(i)
-        elif (t == str) and re.search(r'^[0-9]+\.[0-9]$', i):
-            d[k] = float(i)
-        elif t == dict:
-            d[k] = _type_conversion(i)
-    return d
-
-
 def _handle_get_configuration_not_found_errors(resp):
     # Solace Cloud can return:
     # "error": {
@@ -559,51 +522,6 @@ def get_configuration(solace_config, path_array, key):
 
 # request/response handling
 
-# data_dict = xmltodict.parse(xml_file.read())
-# json_data = json.dumps(data_dict)
-
-def log_http_roundtrip(resp):
-    if hasattr(resp.request, 'body') and resp.request.body:
-        try:
-            decoded_body = resp.request.body.decode()
-            request_body = json.loads(decoded_body)
-        except AttributeError:
-            request_body = resp.request.body
-    else:
-        request_body = "{}"
-
-    if resp.text:
-        try:
-            resp_body = json.loads(resp.text)
-        except JSONDecodeError:
-            # try XML parsing it
-            try:
-                resp_body = xmltodict.parse(resp.text)
-            except Exception:
-                # print as text at least
-                resp_body = resp.text
-    else:
-        resp_body = None
-
-    log = {
-        'request': {
-            'method': resp.request.method,
-            'url': resp.request.url,
-            'headers': dict(resp.request.headers),
-            'body': request_body
-        },
-        'response': {
-            'status_code': resp.status_code,
-            'reason': resp.reason,
-            'url': resp.url,
-            'headers': dict(resp.headers),
-            'body': resp_body
-        }
-    }
-    logging.debug("\n%s", json.dumps(log, indent=2))
-    return
-
-
 def _wait_solace_cloud_request_completed(solace_config, request_resp):
     # GET https://api.solace.cloud/api/v0/services/{paste-your-serviceId-here}/requests/{{requestId}}
     request_resp_body = json.loads(request_resp.text)
@@ -626,30 +544,36 @@ def _wait_solace_cloud_request_completed(solace_config, request_resp):
                         headers={'x-broker-name': solace_config.x_broker},
                         params=None
             )
-            if ENABLE_LOGGING:
-                log_http_roundtrip(resp)
+            if sc.ENABLE_LOGGING:
+                sc.log_http_roundtrip(resp)
             if resp.status_code != 200:
-                raise AnsibleError("GET request status error: {}".format(resp.status_code))
-        except requests.exceptions.ConnectionError as e:
-            raise AnsibleError("GET request status error: {}".format(str(e)))
+                return False, resp
+        except (requests.exceptions.ConnectionError, requests.exception.Timeout) as e:
+            raise AnsibleError("Solace Cloud: GET request status error: {}".format(str(e)))
 
         if resp.text:
             resp_body = json.loads(resp.text)
             is_completed = (resp_body['data']['adminProgress'] == 'completed')
             if is_completed:
-                return resp
+                return True, resp_body
+            else:
+                ok, err = scu.parse_resp_body_for_errs(resp_body)
+                if not ok:
+                    return False, err
         else:
-            raise AnsibleError("GET request status error: no body found in response")
+            raise AnsibleError("Solace Cloud: GET request status error: no body found in response")
         try_count += 1
         time.sleep(delay)
+    # never gets here
+    return True, None
 
 
 def _parse_response(solace_config, resp):
-    if ENABLE_LOGGING:
-        log_http_roundtrip(resp)
+    if sc.ENABLE_LOGGING:
+        sc.log_http_roundtrip(resp)
     # Solace Cloud API returns 202: accepted if long running request
     if resp.status_code == 202 and is_broker_solace_cloud(solace_config):
-        resp = _wait_solace_cloud_request_completed(solace_config, resp)
+        return _wait_solace_cloud_request_completed(solace_config, resp)
     elif resp.status_code != 200:
         return False, parse_bad_response(resp)
     return True, parse_good_response(resp)
@@ -702,17 +626,7 @@ def parse_bad_response(resp):
 
 
 def compose_path(path_array):
-    if not type(path_array) is list:
-        raise TypeError("argument 'path_array' is not an array but {}".format(type(path_array)))
-    # ensure elements are 'url encoded'
-    # except first one: SEMP_V2_CONFIG or SOLACE_CLOUD_API_SERVICES_BASE_PATH
-    paths = []
-    for i, path_elem in enumerate(path_array):
-        if i > 0:
-            paths.append(path_elem.replace('/', '%2F'))
-        else:
-            paths.append(path_elem)
-    return '/'.join(paths)
+    return sc.compose_path(path_array)
 
 
 def compose_solace_cloud_body(operation, type, data):
@@ -761,8 +675,8 @@ def _make_request(func, solace_config, path_array, json=None):
                 params=None
             )
         )
-    except requests.exceptions.ConnectionError as e:
-        logging.debug("ConnectionError: %s", str(e))
+    except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as e:
+        logging.debug("Request Error: %s", str(e))
         return False, str(e)
 
 
