@@ -37,50 +37,32 @@ DOCUMENTATION = '''
 ---
 module: solace_cert_authority
 
-short_description: Configure a certificate authority object.
+short_description: Configure a Certificate Authority object.
 
 description:
-  - "Allows addition, removal and configuration of certificate authority objects on Solace Brokers in an idempotent manner."
-  - "Reference: https://docs.solace.com/API-Developer-Online-Ref-Documentation/swagger-ui/config/index.html#/certAuthority."
+- "Allows addition, removal and configuration of certificate authority objects on Solace Brokers in an idempotent manner."
+
+notes:
+- "Reference: https://docs.solace.com/API-Developer-Online-Ref-Documentation/swagger-ui/config/index.html#/certAuthority."
 
 options:
-  name:
-    description: The name of the Certificate Authority. Maps to 'certAuthorityName' in the API.
-    required: true
-  settings:
-    description: JSON dictionary of additional configuration, see Reference documentation.
-    required: false
-  state:
-    description: Target state. [present|absent].
-    required: false
-    default: present
-  host:
-    description: Hostname of Solace Broker.
-    required: false
-    default: "localhost"
-  port:
-    description: Management port of Solace Broker.
-    required: false
-    default: 8080
-  secure_connection:
-    description: If true, use https rather than http for querying.
-    required: false
-    default: false
-  username:
-    description: Administrator username for Solace Broker.
-    required: false
-    default: "admin"
-  password:
-    description: Administrator password for Solace Broker.
-    required: false
-    default: "admin"
-  timeout:
-    description: Connection timeout in seconds for the http request.
-    required: false
-    default: 1
-  x_broker:
-    description: Custom HTTP header with the broker virtual router id, if using a SEMPv2 Proxy/agent infrastructure.
-    required: false
+    name:
+        description: The name of the Certificate Authority. Maps to 'certAuthorityName' in the API.
+        required: true
+        type: str
+    cert_content:
+        description: The certificate.
+        required: false
+        default: ''
+        type: str
+
+
+extends_documentation_fragment:
+- solace.broker
+- solace.state
+- solace.settings
+
+
 
 
 author:
@@ -156,7 +138,6 @@ class SolaceCertAuthorityTask(su.SolaceTask):
         return su.get_configuration(solace_config, path_array, self.LOOKUP_ITEM_KEY)
 
     def create_func(self, solace_config, cert_content, cert_authority, settings=None):
-        """Create a cert_authority"""
         defaults = {
             'certContent': cert_content
         }
@@ -168,34 +149,27 @@ class SolaceCertAuthorityTask(su.SolaceTask):
         return su.make_post_request(solace_config, path_array, data)
 
     def update_func(self, solace_config, cert_content, lookup_item_value, settings):
-        """Update an existing cert_authority"""
         path_array = [su.SEMP_V2_CONFIG, su.CERT_AUTHORITIES, lookup_item_value]
         return su.make_patch_request(solace_config, path_array, settings)
 
     def delete_func(self, solace_config, cert_content, lookup_item_value):
-        """Delete a cert_authority"""
         path_array = [su.SEMP_V2_CONFIG, su.CERT_AUTHORITIES, lookup_item_value]
         return su.make_delete_request(solace_config, path_array)
 
 
 def run_module():
-    """Entrypoint to module"""
     module_args = dict(
-        name=dict(type='str', required=True),
-        cert_content=dict(type='str', default=''),
-        host=dict(type='str', default='localhost'),
-        port=dict(type='int', default=8080),
-        secure_connection=dict(type='bool', default=False),
-        username=dict(type='str', default='admin'),
-        password=dict(type='str', default='admin', no_log=True),
-        settings=dict(type='dict', require=False),
-        state=dict(default='present', choices=['absent', 'present']),
-        timeout=dict(default='1', require=False),
-        x_broker=dict(type='str', default='')
+        cert_content=dict(type='str', default='')
     )
+    arg_spec = su.arg_spec_broker()
+    arg_spec.update(su.arg_spec_settings())
+    arg_spec.update(su.arg_spec_state())
+    arg_spec.update(su.arg_spec_name())
+    # module_args override standard arg_specs
+    arg_spec.update(module_args)
 
     module = AnsibleModule(
-        argument_spec=module_args,
+        argument_spec=arg_spec,
         supports_check_mode=True
     )
 
@@ -206,7 +180,7 @@ def run_module():
 
 
 def main():
-    """Standard boilerplate"""
+
     run_module()
 
 
