@@ -30,10 +30,11 @@ ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
-import ansible.module_utils.network.solace.solace_utils as su
+import ansible.module_utils.network.solace.solace_common as sc
 from ansible.module_utils.basic import AnsibleModule
 import traceback
 HAS_IMPORT_ERROR = False
+IMPORT_ERR_TRACEBACK = None
 try:
     from ansible.errors import AnsibleError
     from urllib.parse import urlparse
@@ -86,12 +87,16 @@ options:
       - get_virtualRouterName
       - get_serviceSMFMessagingEndpoints
       - get_bridge_remoteMsgVpnLocations
+      - get_allClientConnectionDetails
     functions:
       get_bridge_remoteMsgVpnLocations:
         description:
-            - "Retrieve remote message vpn locations (plain, secured, compressed) for the service/broker."
+            - "Retrieve enabled remote message vpn locations (plain, secured, compressed) for the service/broker."
             - "For Solace Cloud: {hostname}:{port}."
             - "For broker: v:{virtualRouterName}."
+      get_allClientConnectionDetails:
+        description:
+            - "Retrieve all enabled client connection details for the various protocols for the service/broker."
 
 seealso:
 - module: solace_gather_facts
@@ -199,10 +204,7 @@ facts:
 class SolaceGetFactsTask():
 
     def __init__(self, module):
-        if HAS_IMPORT_ERROR:
-            exceptiondata = traceback.format_exc().splitlines()
-            exceptionarray = [exceptiondata[-1]] + exceptiondata[1:-1]
-            module.fail_json(msg="failed: Missing module: %s" % exceptionarray[0], exception=IMPORT_ERR_TRACEBACK)
+        sc.module_fail_on_import_error(module, HAS_IMPORT_ERROR, IMPORT_ERR_TRACEBACK)
         self.module = module
         return
 
@@ -210,6 +212,7 @@ class SolaceGetFactsTask():
 
         hostvars = self.module.params['hostvars']
         host = self.module.params['host']
+        vpn = self.module.params['msg_vpn']
         fields = self.module.params['fields']
         field_funcs = self.module.params['field_funcs']
         # either fields or field_funcs must have at least one element
@@ -235,6 +238,10 @@ class SolaceGetFactsTask():
 
         search_object = hostvars[host]['ansible_facts']['solace']
 
+        if not _check_vpn_exists(search_object, vpn):
+            fail_reason = "Could not find vpn: '{}' in 'ansible_facts.solace' for host: '{}'.".format(vpn, host)
+            return False, fail_reason
+
         facts = dict()
 
         if fields is not None and len(fields) > 0:
@@ -260,6 +267,8 @@ class SolaceGetFactsTask():
                         field, value = _get_serviceSMFMessagingEndpoints(search_object)
                     elif field_func == 'get_bridge_remoteMsgVpnLocations':
                         field, value = _get_bridge_remoteMsgVpnLocations(search_object)
+                    elif field_func == 'get_allClientConnectionDetails':
+                        field, value = _get_allClientConnectionDetails(search_object, vpn)
                     else:
                         fail_reason = "Unknown field_func: '{}'. Pls check the documentation for supported field functions: 'ansible-doc solace_get_facts'.".format(field_func)
                         return False, fail_reason
@@ -285,6 +294,63 @@ class SolaceGetFactsTask():
 #
 
 
+def _check_vpn_exists(search_dict, search_vpn):
+    if not search_vpn:
+        return True
+    if search_dict['isSolaceCloud']:
+        message_vpn_attributes_dict = _get_sc_message_vpn_attributes_dict(search_dict)
+        vpn = message_vpn_attributes_dict['vpnName']
+        return (vpn == search_vpn)
+    else:
+        # for SEMPv1:
+        vpns = search_dict['about']['user']['msgVpns']
+        for vpn in vpns:
+            if vpn['msgVpnName'] == search_vpn:
+                return True
+        # TODO: for SEMPv2:
+        # once solace_gather_facts is extended
+    return False
+
+
+def _get_allClientConnectionDetails(search_dict, vpn=None):
+    ccds = dict()
+    if search_dict['isSolaceCloud']:
+        # msg_vpn_name = _get_field(search_dict, field='msgVpnName'):
+        # TODO: needs to find it for all vpns: "vpn-name": "default",
+        smf_dict = _get_sc_messaging_protocol_dict(search_dict, 'SMF')
+        mqtt_dict = _get_sc_messaging_protocol_dict(search_dict, 'MQTT')
+        amqp_dict = _get_sc_messaging_protocol_dict(search_dict, 'AMQP')
+        rest_dict = _get_sc_messaging_protocol_dict(search_dict, 'REST')
+        jms_dict = _get_sc_messaging_protocol_dict(search_dict, 'JMS')
+        web_msg_dict = _get_sc_messaging_protocol_dict(search_dict, 'Web Messaging')
+        message_vpn_attributes_dict = _get_sc_message_vpn_attributes_dict(search_dict)
+        trust_store_uri = message_vpn_attributes_dict['truststoreUri']
+    else:
+        # logging.debug("\n\n broker: search_dict=\n%s\n\n", json.dumps(search_dict, indent=2))
+        # TODO: needs to find it for all vpns: "vpn-name": "default",
+        # TODO: only retrieve if vpn-name exists (relevant for AMQP)
+
+        smf_dict = _get_broker_service_dict(search_dict, field="name", value='SMF', strict=False)
+        mqtt_dict = _get_broker_service_dict(search_dict, field="name", value='MQTT', strict=False)
+        amqp_dict = _get_broker_service_dict(search_dict, field="name", value='AMQP', strict=False)
+        rest_dict = _get_broker_service_dict(search_dict, field="name", value='REST', strict=False)
+        # using SEMPv1: assuming same as SMF, check with SEMPv2
+        # jms_dict = _get_broker_service_dict(search_dict, field="name", value='JMS', strict=False)
+        jms_dict = None
+        web_msg_dict = _get_broker_service_dict(search_dict, field="name", value='WEB', strict=False)
+        trust_store_uri = None
+
+    ccds['SMF'] = smf_dict
+    ccds['MQTT'] = mqtt_dict
+    ccds['AMQP'] = amqp_dict
+    ccds['REST'] = rest_dict
+    ccds['JMS'] = jms_dict
+    ccds['WebMessaging'] = web_msg_dict
+    if trust_store_uri:
+        ccds['TrustStore'] = dict(uri=trust_store_uri)
+    return 'clientConnectionDetails', ccds
+
+
 def _get_bridge_remoteMsgVpnLocations(search_dict):
     locs = dict(
         plain=None,
@@ -293,12 +359,21 @@ def _get_bridge_remoteMsgVpnLocations(search_dict):
     )
     if search_dict['isSolaceCloud']:
         f, smfMessagingEndpoints = _get_serviceSMFMessagingEndpoints(search_dict)
-        locs['plain'] = (smfMessagingEndpoints['SMF']['SMF']['uriComponents']['host']
-                         + ":" + str(smfMessagingEndpoints['SMF']['SMF']['uriComponents']['port']))
-        locs['compressed'] = (smfMessagingEndpoints['SMF']['CompressedSMF']['uriComponents']['host']
-                              + ":" + str(smfMessagingEndpoints['SMF']['CompressedSMF']['uriComponents']['port']))
-        locs['secured'] = (smfMessagingEndpoints['SMF']['SecuredSMF']['uriComponents']['host']
-                           + ":" + str(smfMessagingEndpoints['SMF']['SecuredSMF']['uriComponents']['port']))
+        if smfMessagingEndpoints['SMF']['SMF']['uriComponents']['host']:
+            locs['plain'] = (str(smfMessagingEndpoints['SMF']['SMF']['uriComponents']['host'])
+                             + ":" + str(smfMessagingEndpoints['SMF']['SMF']['uriComponents']['port']))
+        else:
+            locs['plain'] = None
+        if smfMessagingEndpoints['SMF']['CompressedSMF']['uriComponents']['host']:
+            locs['compressed'] = (str(smfMessagingEndpoints['SMF']['CompressedSMF']['uriComponents']['host'])
+                                  + ":" + str(smfMessagingEndpoints['SMF']['CompressedSMF']['uriComponents']['port']))
+        else:
+            locs['compressed'] = None
+        if smfMessagingEndpoints['SMF']['SecuredSMF']['uriComponents']['host']:
+            locs['secured'] = (str(smfMessagingEndpoints['SMF']['SecuredSMF']['uriComponents']['host'])
+                               + ":" + str(smfMessagingEndpoints['SMF']['SecuredSMF']['uriComponents']['port']))
+        else:
+            locs['secured'] = None
     else:
         f, virtual_router = _get_virtualRouterName(search_dict)
         loc = "v:" + virtual_router
@@ -317,39 +392,42 @@ def _get_serviceSMFMessagingEndpoints(search_dict):
             CompressedSMF=dict()
         )
     )
+    smf_protocol = None
+    smf_host = None
+    smf_port = None
+    smf_uri = None
+
+    sec_smf_protocol = None
+    sec_smf_host = None
+    sec_smf_port = None
+    sec_smf_uri = None
+
+    cmp_smf_protocol = None
+    cmp_smf_host = None
+    cmp_smf_port = None
+    cmp_smf_uri = None
+
     if search_dict['isSolaceCloud']:
         smf_dict = _get_sc_messaging_protocols_smf_dict(search_dict)
+        # if endPoint is not enabled, API omits it
         smf_end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='SMF')
+        if smf_end_point_dict:
+            smf_uri = _get_sc_messaging_protocol_endpoint_uri(smf_end_point_dict)
+            t = urlparse(smf_uri)
+            smf_protocol = t.scheme
+            smf_host = t.hostname
         sec_smf_end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='Secured SMF')
+        if sec_smf_end_point_dict:
+            sec_smf_uri = _get_sc_messaging_protocol_endpoint_uri(sec_smf_end_point_dict)
+            t = urlparse(sec_smf_uri)
+            sec_smf_protocol = t.scheme
+            sec_smf_host = t.hostname
         cmp_smf_end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='Compressed SMF')
-
-        smf_uri = _get_sc_messaging_protocol_endpoint_uri(smf_end_point_dict)
-        t = urlparse(smf_uri)
-        smf_protocol = t.scheme
-        smf_host = t.hostname
-
-        sec_smf_uri = _get_sc_messaging_protocol_endpoint_uri(sec_smf_end_point_dict)
-        t = urlparse(sec_smf_uri)
-        sec_smf_protocol = t.scheme
-        sec_smf_host = t.hostname
-
-        cmp_smf_uri = _get_sc_messaging_protocol_endpoint_uri(cmp_smf_end_point_dict)
-        t = urlparse(cmp_smf_uri)
-        cmp_smf_protocol = t.scheme
-        cmp_smf_host = t.hostname
-    else:
-        # smf_dict = _get_broker_service_dict(search_dict, field="name", value="SMF")
-        smf_protocol = None
-        smf_host = None
-        smf_uri = None
-
-        sec_smf_protocol = None
-        sec_smf_host = None
-        sec_smf_uri = None
-
-        cmp_smf_protocol = None
-        cmp_smf_host = None
-        cmp_smf_uri = None
+        if cmp_smf_end_point_dict:
+            cmp_smf_uri = _get_sc_messaging_protocol_endpoint_uri(cmp_smf_end_point_dict)
+            t = urlparse(cmp_smf_uri)
+            cmp_smf_protocol = t.scheme
+            cmp_smf_host = t.hostname
 
     f, smf_port = _get_serviceSmfPlainTextListenPort(search_dict)
     f, sec_smf_port = _get_serviceSmfTlsListenPort(search_dict)
@@ -389,8 +467,11 @@ def _get_serviceSmfPlainTextListenPort(search_dict):
     if search_dict['isSolaceCloud']:
         smf_dict = _get_sc_messaging_protocols_smf_dict(search_dict)
         end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='SMF')
-        uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
-        value = _get_port_from_uri(uri)
+        if end_point_dict:
+            uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
+            value = _get_port_from_uri(uri)
+        else:
+            value = None
     else:
         smf_dict = _get_broker_service_dict(search_dict, field="name", value="SMF")
         value = smf_dict['listen-port']
@@ -401,8 +482,11 @@ def _get_serviceSmfCompressionListenPort(search_dict):
     if search_dict['isSolaceCloud']:
         smf_dict = _get_sc_messaging_protocols_smf_dict(search_dict)
         end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='Compressed SMF')
-        uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
-        value = _get_port_from_uri(uri)
+        if end_point_dict:
+            uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
+            value = _get_port_from_uri(uri)
+        else:
+            value = None
     else:
         smf_dict = _get_broker_service_dict(search_dict, field="name", value="SMF")
         value = smf_dict['compression-listen-port']
@@ -413,8 +497,11 @@ def _get_serviceSmfTlsListenPort(search_dict):
     if search_dict['isSolaceCloud']:
         smf_dict = _get_sc_messaging_protocols_smf_dict(search_dict)
         end_point_dict = _get_sc_messaging_protocol_endpoint(smf_dict, field='name', value='Secured SMF')
-        uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
-        value = _get_port_from_uri(uri)
+        if end_point_dict:
+            uri = _get_sc_messaging_protocol_endpoint_uri(end_point_dict)
+            value = _get_port_from_uri(uri)
+        else:
+            value = None
     else:
         smf_dict = _get_broker_service_dict(search_dict, field="name", value="SMF")
         value = smf_dict['ssl']['listen-port']
@@ -433,13 +520,24 @@ def _get_virtualRouterName(search_dict):
 # field func helpers
 #
 
-def _get_broker_service_dict(search_dict, field, value):
+def _get_broker_service_dict(search_dict, field, value, strict=True):
     service_dict = _find_nested_dict(search_dict, field, value)
     if service_dict is None:
-        raise AnsibleError("Could not find '{}={}' in search_dict in broker service ansible_facts. Pls raise an issue.".format(field, value))
+        if strict:
+            raise AnsibleError("Could not find '{}={}' in search_dict in broker service ansible_facts. Pls raise an issue.".format(field, value))
+        else:
+            service_dict = dict(enabled=False)
     return service_dict
 
 
+def _get_sc_message_vpn_attributes_dict(search_dict):
+    element = "msgVpnAttributes"
+    message_vpn_attributes_dict = _get_field(search_dict, element)
+    if message_vpn_attributes_dict is None:
+        raise AnsibleError("Could not find '{}' in Solace Cloud service ansible_facts. API may have changed. Pls raise an issue.".format(element))
+    return message_vpn_attributes_dict
+
+
 def _get_sc_messaging_protocols_dict(search_dict):
     element = "messagingProtocols"
     messaging_protocols_dict = _get_field(search_dict, element)
@@ -448,12 +546,24 @@ def _get_sc_messaging_protocols_dict(search_dict):
     return messaging_protocols_dict
 
 
+def _get_sc_messaging_protocol_dict(search_dict, protocol):
+    messaging_protocols_dict = _get_sc_messaging_protocols_dict(search_dict)
+    protocol_dict = _find_nested_dict(messaging_protocols_dict, field="name", value=protocol)
+    if protocol_dict is None:
+        protocol_dict = dict(
+            enabled=False
+        )
+    else:
+        protocol_dict['enabled'] = True
+    return protocol_dict
+
+
 def _get_sc_messaging_protocols_smf_dict(search_dict):
     messaging_protocols_dict = _get_sc_messaging_protocols_dict(search_dict)
     search_value = 'SMF'
     smf_dict = _find_nested_dict(messaging_protocols_dict, field="name", value=search_value)
     if smf_dict is None:
-        raise AnsibleError("Could not find 'name={}' in messaging protocols in Solace Cloud service ansible_facts. API may have changed. Pls raise an issue.".format(search_value))
+        raise AnsibleError("Could not find 'name={}' in messaging protocols in Solace Cloud service ansible_facts. Check if it is enabled.".format(search_value))
     return smf_dict
 
 
@@ -465,8 +575,10 @@ def _get_sc_messaging_protocol_endpoint(search_dict, field, value):
     if len(end_points) == 0:
         raise AnsibleError("List:'{}' in dict:{} in Solace Cloud service ansible_facts. API may have changed. Pls raise an issue.".format(element, json.dumps(search_dict)))
     end_point_dict = _find_nested_dict(end_points, field, value)
-    if end_point_dict is None:
-        raise AnsibleError("Could not find messaging protocol end point with '{}={}' in Solace Cloud service ansible_facts. API may have changed. Pls raise an issue.".format(field, value))
+    # endPoint may not be enabled
+    # if end_point_dict is None:
+    #     # might not be enabled
+    #     raise AnsibleError("Could not find messaging protocol end point with '{}={}' in Solace Cloud service ansible_facts. Check if it is enabled.".format(field, value))
     return end_point_dict
 
 
@@ -476,9 +588,10 @@ def _get_sc_messaging_protocol_endpoint_uri(search_dict):
         errs = [
             "Could not find '{}' in messaging protocol end point:".format(element),
             search_dict,
-            "in Solace Cloud service ansible_facts. API may have changed. Pls raise an issue."
+            "in Solace Cloud service ansible_facts."
         ]
-        raise AnsibleError(json.dumps(errs))
+        return dict()
+        # raise AnsibleError(json.dumps(errs))
     if len(search_dict['uris']) != 1:
         errs = [
             "'{}' list contains != 1 elements in messaging protocol end point:".format(element),
@@ -531,9 +644,12 @@ def run_module():
         hostvars=dict(type='dict', required=True),
         host=dict(type='str', required=True),
         fields=dict(type='list', required=False, default=[], elements='str'),
-        field_funcs=dict(type='list', required=False, default=[], elements='str')
+        field_funcs=dict(type='list', required=False, default=[], elements='str'),
+        msg_vpn=dict(type='str', required=False, default=None)
     )
-    arg_spec = su.arg_spec_solace_cloud_config()
+    arg_spec = dict()
+    # TODO: check if we need the msg_vpn with multi-vpn brokers
+    # arg_spec = su.arg_spec_vpn()
     # module_args override standard arg_specs
     arg_spec.update(module_args)
 
